#if !defined IMAGEPROCESSOR
#define IMAGEPROCESSOR

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <streams.h>
#include <dxutil.h>
#include <qedit.h>
#include <vector>

#include <iProxyTrans.h>
#include <ProxyTransuids.h>

#include "cv.h"      // include core library interface
#include "highgui.h" // include GUI library interface
#include "filters.h"

void process(void* img);


class SequenceProcessor {
 
    IGraphBuilder *pGraph;
    IMediaControl *pMediaControl;
    WCHAR* ifilename;
    WCHAR* ofilename;
	CString compressorName;
	CString cameraName;
	int keyFrames, pFrames;
	double quality;

  public:

    bool createFilterGraph() {

      IPin* pSourceOut[2];
      pSourceOut[0]= pSourceOut[1]= NULL;
     
	  // Video source
	  if (cameraName == "") {

        addSource(ifilename, pGraph, pSourceOut);

	  } else {

		addFilterByEnum(CLSID_VideoInputDeviceCategory,cameraName,pGraph,pSourceOut,1);
	  }
      
	  if (cameraName == "") {

  	    // Add the decoding filters
        addFilter(CLSID_AviSplitter, L"Splitter", pGraph, pSourceOut);
        addFilter(CLSID_AVIDec, L"Decoder", pGraph, pSourceOut);
	  
        // Insert the first Smart Tee
        addFilter(CLSID_SmartTee, L"SmartTee(1)", pGraph, pSourceOut,2);
	  }

      // Add the ProxyTrans filter
	  addFilter(CLSID_ProxyTransform, L"ProxyTrans", pGraph, pSourceOut);

      // Set the ProxyTrans callback     
	  IBaseFilter* pProxyFilter = NULL;
	  IProxyTransform* pProxyTrans = NULL;
      pGraph->FindFilterByName(L"ProxyTrans",&pProxyFilter);
      pProxyFilter->QueryInterface(IID_IProxyTransform, (void**)&pProxyTrans);
      pProxyTrans->set_transform(process, 0);
      SAFE_RELEASE(pProxyTrans);
      SAFE_RELEASE(pProxyFilter);

	  if (cameraName == "") {
		// Render the original (decoded) sequence using 2nd SmartTee(1) output pin
		addRenderer(L"Renderer(1)", pGraph, pSourceOut+1);
	  }

      // Insert the second Smart Tee
	  if (ofilename != 0)
		addFilter(CLSID_SmartTee, L"SmartTee(2)", pGraph, pSourceOut,2);

	  // Encode the processed sequence 
	  if (compressorName != "") {

		addFilterByEnum(CLSID_VideoCompressorCategory,compressorName,pGraph,pSourceOut);
        changeCompressorOptions(pSourceOut, 1, keyFrames, pFrames, quality);
	  }

	  if (ofilename != 0) {

        addFilter(CLSID_AviDest, L"AVImux", pGraph, pSourceOut);
		// Write to file
		addFileWriter(ofilename, pGraph, pSourceOut);
		// Render the transformed sequence using 2nd SmartTee(2) output pin
		addRenderer(L"Renderer(2)", pGraph, pSourceOut+1);

	  } else {

		// Render the transformed sequence 
		addRenderer(L"Renderer(2)", pGraph, pSourceOut);
	  }
      
      return 1;
    }
    
    SequenceProcessor() {

      keyFrames= pFrames= -1;
	  quality= -1.0;

      ifilename=0;
      ofilename=0;
	  compressorName= "";
	  cameraName= "";

      CoInitialize(NULL);

      pGraph= NULL;
      pMediaControl= NULL;


      if (!FAILED(
             CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC, 
                              IID_IGraphBuilder, (void **)&pGraph))) {

        pGraph->QueryInterface(IID_IMediaControl, (void **)&pMediaControl);

      }
    }

    void setInput(CString inputfilename) {

	  cameraName= "";
      delete[] ifilename;
      WCHAR	*mediaFile= new WCHAR[inputfilename.GetLength()+1];
      MultiByteToWideChar(CP_ACP, 0, inputfilename, -1, mediaFile, inputfilename.GetLength()+1);
      ifilename= mediaFile;
    }

    void setOutput(CString outputfilename) {

      delete[] ofilename;
      WCHAR	*mediaFile= new WCHAR[outputfilename.GetLength()+1];
      MultiByteToWideChar(CP_ACP, 0, outputfilename, -1, mediaFile, outputfilename.GetLength()+1);
      ofilename= mediaFile;
    }

	void setCompressor(CString cn, int key, int pf, double q) {

	  compressorName= cn;
	  keyFrames= key;
	  pFrames= pf;
	  quality= q;
	}

	void setCamera(CString cn) {

	  cameraName= cn;
	}

	void setCameraOptions(HWND hwndParent) {

		if (cameraName == "")
			return;

		IAMVfwCaptureDialogs *pVfw = 0;
		IBaseFilter*pCap= getFilterByName(pGraph, cameraName);

		if (pCap!=0 && SUCCEEDED(pCap->QueryInterface(IID_IAMVfwCaptureDialogs, (void**)&pVfw))) {

			// Check if the device supports this dialog box.
			if (S_OK == pVfw->HasDialog(VfwCaptureDialog_Format)) {

				// Show the dialog box.
				pVfw->ShowDialog(VfwCaptureDialog_Format, hwndParent);
			}
    
			if (S_OK == pVfw->HasDialog(VfwCaptureDialog_Source)) {

				// Show the dialog box.
				pVfw->ShowDialog(VfwCaptureDialog_Source, hwndParent);
			}	

			SAFE_RELEASE(pVfw);
		}

        SAFE_RELEASE(pCap);
	}

	void getCompressorOptions(int& key, int& pf, double& q) {

	  key= keyFrames;
	  pf= pFrames;
	  q= quality;
	}

    void run() {

      pMediaControl->Run();
    }

    void stop() {

      pMediaControl->Stop();
    }

    void clear() {

      if (pGraph)
       removeAllFilters(pGraph);
    }

    std::vector<CString> enumFilters() {

      return ::enumFilters(pGraph);
    }

    ~SequenceProcessor() {


      delete[] ofilename;
      delete[] ifilename;
      SAFE_RELEASE(pMediaControl);
      SAFE_RELEASE(pGraph);

      CoUninitialize();
    }
};


#endif

